#include "../../../include/asset_mgr.hpp"
#include "../../../include/shared.hpp"
#include "include/GTE_Sprite.hpp"
#include "include/gte_test_assets.hpp"
#include <PSX/Periphery/periphery.hpp>

namespace GTETest {
    using namespace JabyEngine;
    using namespace GTETest;

    static auto jaby_star_base = Make::SPRT(
        // TODO: improve magic numbers
        Make::AreaI16(GPU::Display::Width - 64, GPU::Display::Height - 64, 64, 64), 
        Make::OffsetPageWithClut(JabySTARTim.get_page_offset_clut4(), Make::PageClut(JabySTARTim.get_clut_position()))
    );

    static auto doener_fish = GTE_Sprite::create(Make::POLY_FT4(
        Make::AreaI16(Make::PositionI16(0, 0), Assets::Main::DoenerFishInfo.size),
        Assets::Main::DoenerFishInfo.tim.get_page_offset_clut4(),
        Make::TPage(Assets::Main::DoenerFishInfo.tim.get_texture_position(), GPU::SemiTransparency::B_add_F, GPU::TextureColorMode::clut4),
        Make::PageClut(Assets::Main::DoenerFishInfo.tim.get_clut_position()),
        GPU::Color24::Grey()
    ));

    static auto gbl_rotation = 0.0_deg;

    static void setup() {
        doener_fish.area.position = GPU::PositionI16::create(100, 100);
        Shared::back_menu.reset();

        GTE::set_geom_offset(0, 0);
        GTE::set_geom_screen(512);
    }

    static bool update_or_exit() {
        Periphery::query_controller();
        if(Shared::back_menu.update(Make::PositionI16(0, GPU::Display::Height - 32))) {
            return true;
        }

        auto matrix = GTE::MATRIX::rotated(-gbl_rotation, gbl_rotation, -gbl_rotation);
        doener_fish.apply(matrix);
        doener_fish.angle += 25.0_deg;
        gbl_rotation      += 2.5_deg;
        return false;
    }
    
    static void render() {
        doener_fish.render();
        GPU::render(jaby_star_base);
        Shared::back_menu.render();
    }

    void main() {
        setup();

        while(true) {
            if(update_or_exit()) {
                break;
            }
            GPU::swap_buffers_vsync(1);
            render();
        }
    }
}